@@ -1,7 +1,7 @@
 package App::rimetadb;
 
 our $DATE = '2015-12-01'; # DATE
-our $VERSION = '0.13'; # VERSION
+our $VERSION = '0.14'; # VERSION
 
 use 5.010001;
 use strict;
@@ -12,10 +12,13 @@ use Log::Any::IfLOG '$log';
 use Module::Load qw(autoload load);
 
 our $db_schema_spec = {
-    latest_v => 4,
+    latest_v => 5,
     install => [
         'CREATE TABLE IF NOT EXISTS package (name VARCHAR(255) PRIMARY KEY, summary TEXT, metadata BLOB, dist TEXT, extra TEXT, mtime INT)',
-        'CREATE TABLE IF NOT EXISTS function (package VARCHAR(255) NOT NULL, name VARCHAR(255) NOT NULL, summary TEXT, metadata BLOB, dist TEXT, extra TEXT, UNIQUE(package, name))',
+        'CREATE TABLE IF NOT EXISTS function (package VARCHAR(255) NOT NULL, name VARCHAR(255) NOT NULL, summary TEXT, metadata BLOB, dist TEXT, extra TEXT, mtime INT, UNIQUE(package, name))',
+    ],
+    upgrade_to_v5 => [
+        'ALTER TABLE function ADD COLUMN mtime INT',
     ],
     upgrade_to_v4 => [
         'ALTER TABLE function ADD COLUMN dist TEXT',
@@ -37,6 +40,10 @@ our $db_schema_spec = {
     ],
 
     # for testing
+    install_v4 => [
+        'CREATE TABLE IF NOT EXISTS package (name VARCHAR(255) PRIMARY KEY, summary TEXT, metadata BLOB, dist TEXT, extra TEXT, mtime INT)',
+        'CREATE TABLE IF NOT EXISTS function (package VARCHAR(255) NOT NULL, name VARCHAR(255) NOT NULL, summary TEXT, metadata BLOB, dist TEXT, extra TEXT, UNIQUE(package, name))',
+    ],
     install_v3 => [
         'CREATE TABLE IF NOT EXISTS package (name VARCHAR(255) PRIMARY KEY, summary TEXT, metadata BLOB, dist TEXT, extra TEXT, mtime INT)',
         'CREATE TABLE IF NOT EXISTS function (package VARCHAR(255) NOT NULL, name VARCHAR(255) NOT NULL, summary TEXT, metadata BLOB, extra TEXT, UNIQUE(package, name))',
@@ -424,12 +431,12 @@ sub update {
     if ($func) {
         my $funcsummary = $meta->{summary};
         if ($dbh->selectrow_array("SELECT name FROM function WHERE package=? AND name=?", {}, $pkg, $func)) {
-            $dbh->do("UPDATE function SET summary=?, metadata=?, mtime=?, dist=?, extra=? WHERE package=? AND function=?",
+            $dbh->do("UPDATE function SET summary=?, metadata=?, mtime=?, dist=?, extra=? WHERE package=? AND name=?",
                      {}, $funcsummary, _json->encode($meta), time(), $args{dist}, $args{extra},
                      $pkg, $func);
         } else {
-            $dbh->do("INSERT INTO function (package, name, summary, metadata, dist, extra) VALUES (?,?,?,?,?)",
-                     {}, $pkg, $func, $funcsummary, _json->encode($meta), $args{dist}, $args{extra});
+            $dbh->do("INSERT INTO function (package, name, summary, metadata, mtime, dist, extra) VALUES (?,?,?,?,?,?,?)",
+                     {}, $pkg, $func, $funcsummary, _json->encode($meta), time(), $args{dist}, $args{extra});
         }
     }
 
@@ -528,7 +535,7 @@ sub functions {
     my $q  = $args{query};
 
     my @rows;
-    my @columns = qw(package name summary dist extra);
+    my @columns = qw(package name summary dist mtime extra);
     my @wheres;
     my @binds;
 
@@ -538,7 +545,7 @@ sub functions {
     }
 
     my $sth = $dbh->prepare(
-        "SELECT package,name,summary,dist,extra FROM function ORDER by package,name".
+        "SELECT package,name,summary,dist,mtime,extra FROM function ORDER by package,name".
             (@wheres ? " WHERE ".join(" AND ", @wheres) : "")
     );
     $sth->execute(@binds);
@@ -547,7 +554,7 @@ sub functions {
         if ($args{detail}) {
             push @rows, $row;
         } else {
-            push @rows, $row->{name};
+            push @rows, "$row->{package}\::$row->{name}";
         }
     }
 
@@ -593,7 +600,7 @@ App::rimetadb - Manage a Rinci metadata database
 
 =head1 VERSION
 
-This document describes version 0.13 of App::rimetadb (from Perl distribution App-rimetadb), released on 2015-12-01.
+This document describes version 0.14 of App::rimetadb (from Perl distribution App-rimetadb), released on 2015-12-01.
 
 =head1 SYNOPSIS
 
